 !
 !  NAME:
 !  	spica_struct_1a_mod_type.f
 !
 !  PURPOSE:
 !	This module contains the declarations of the different derived data types
 ! 	specific to the SPICAM/SPICAV UV/IR data level 1A structure 
 !
 ! Author:
 !   Nicolas Chapron
 !     CNRS | LATMOS - Laboratoire Atmospheres, Milieux, Observations Spatiales.
 !     11 boulevard d'Alembert
 !     78280 Guyancourt
 !     France
 !     Tel. 33-(0) 1 80 28 50 57
 !     Email. nicolas.chapron@latmos.ipsl.fr
 !
 !  SVN Revision:
 !      $Rev: 2365 $
 !      $Author: chapron $
 !      $Date: 2014-10-30 16:01:18 +0100 (Jeu, 30 oct 2014) $
 !
 
module spica_struct_1a_mod_type
    implicit none

! ----- Structures declaration -----------------------------------------------------------------------------------------------------------------------------
! ----------------------------------------------------------------------------------------------------------------------------------------------------------

    ! ----- struc_Info -----------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Info
    	
    	! UV/IR
        character(len=10) instrument
        character(len=1) obstype
        character(len=128) begins
        character(len=128) ends
        integer orbit
        integer seq
        integer naxis1
        integer naxis2
        integer naxis3

        ! UV ONLY
        integer DCProc
        integer ENProc
        integer COSProc
        integer SATProc
        integer orbDCNU
        character(len=128) File0C
        character(len=128) Data_status
	character(len=128) Geo_status
	character(len=128) Flag_status
	character(len=128) DC_status
	
	! IR ONLY
	character(len=128) status
	integer nb_spectrum
        integer nb_points
        integer nb_channels
        integer nb_blocks
        integer size_block

    END TYPE struc_Info
    ! ----- END struc_Info -----------------------------------------------------------------------------------------------------------------------
    
    
    ! ----- struc_Parameters ---------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Parameters
        
    	! UV/IR 
    	integer peltier
        integer soiron

    	! UV ONLY
    	integer codeop
        integer binning
        integer ht
        integer ti
        integer x0
        integer y0
        integer slit
        integer uvsampling
        integer iron
        integer, dimension(:), allocatable :: all_ti
        real, dimension(:), allocatable :: t_peltier
        real, dimension(:), allocatable :: t_ccd
        real, dimension(:), allocatable :: t_numboard
        real, dimension(:), allocatable :: t_btboard
        real, dimension(:), allocatable :: t_shutter
        real, dimension(:), allocatable :: t_servboard
        real, dimension(:), allocatable :: t_hvps
        real, dimension(:), allocatable :: t_structure
        
        ! IR ONLY
        integer uvon
        integer exit
        integer source
        integer gain
        integer dac
        integer dots
        integer ndots
	real time
        real gainboost
        integer dets
        real, dimension(:), allocatable :: frequency
        real, dimension(:), allocatable :: t_d0
        real, dimension(:), allocatable :: t_d1
        real, dimension(:), allocatable :: t_aotf
        real, dimension(:), allocatable :: t_base
        real, dimension(:), allocatable :: power_rf
        real, dimension(:), allocatable :: pvs
    
    END TYPE struc_Parameters
    ! ----- END struc_Parameters -----------------------------------------------------------------------------------------------------------------
    
    
    ! ----- struc_Time_of_Records ----------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Time_of_Records
    
    	! IR ONLY
        integer, dimension(:), allocatable :: year
        integer, dimension(:), allocatable :: month
        integer, dimension(:), allocatable :: day
        integer, dimension(:), allocatable :: hour
        integer, dimension(:), allocatable :: minute
        integer, dimension(:), allocatable :: second
        real, dimension(:), allocatable :: msecond
    	
    END TYPE struc_Time_of_Records
    ! ----- END struc_Time_of_Records ------------------------------------------------------------------------------------------------------------

    
    ! ----- struc_GeoInfo ------------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_GeoInfo
        
    	! UV/IR
    	character(len=16) target
        real sunlat
        real sunlong
        real sundist
        real sunls
        real sunra
        real sundec
        real, dimension(2) :: slitcenter
        
	! UV ONLY    	
        character(len=16) shadowcone
        
        ! IR ONLY
        real, dimension(2) :: irfov_pos
        real irshift

    END TYPE struc_GeoInfo
    ! ----- END strucGeoInfo --------------------------------------------------------------------------------------------------------------------

    
    ! ----- struc_Geo_Record ---------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_Record
    
    	! UV/IR
        integer, dimension(:), allocatable :: number
        character(len=128), dimension(:), allocatable :: time
    
        END TYPE struc_Geo_Record
    ! ----- END struc_Geo_Record -----------------------------------------------------------------------------------------------------------------
   
    
    ! ----- struc_Geo_Spacecraft -----------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_Spacecraft
    
    	! UV/IR
        real, dimension(:), allocatable ::  lat
        real, dimension(:), allocatable ::  long
        real, dimension(:), allocatable ::  alt
        real, dimension(:), allocatable ::  sza
        
        ! UV ONLY
        real, dimension(:), allocatable ::  shadowconeratio
        
    END TYPE struc_Geo_Spacecraft
    ! ----- END struc_Geo_Spacecraft -------------------------------------------------------------------------------------------------------------

    
    ! ----- struc_Geo_Band3LOSE ------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_Band3LOSE
        
    	! UV ONLY
    	real, dimension(:), allocatable :: sza
        real, dimension(:), allocatable :: disttoplanetnp
        real, dimension(:), allocatable :: pixelsize
        real, dimension(:), allocatable :: anglelossun
        real, dimension(:), allocatable :: phaselossun
        real, dimension(:), allocatable :: solarincidence
        real, dimension(:), allocatable :: solarlocaltime
        real, dimension(:), allocatable :: anglenormalobserver
        real, dimension(:), allocatable :: sunazimuth
        real, dimension(:), allocatable :: distlosplanetcenter
        real, dimension(:), allocatable :: distscplanetcenter
        real, dimension(:), allocatable :: lat
        real, dimension(:), allocatable :: long
        real, dimension(:), allocatable :: alt
        real, dimension(:), allocatable :: ra
        real, dimension(:), allocatable :: dec
    
    END TYPE struc_Geo_Band3LOSE
    ! ----- END struc_Geo_Band3LOSE --------------------------------------------------------------------------------------------------------------

    
    ! ----- struc_Geo_BandCCDline ----------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_BandCCDline
        
    	! UV ONLY
    	real, dimension(:), allocatable :: lat
        real, dimension(:), allocatable :: long
        real, dimension(:), allocatable :: alt
        real, dimension(:), allocatable :: ra
        real, dimension(:), allocatable :: dec
    
    END TYPE struc_Geo_BandCCDline
    ! ----- END struc_Geo_BandCCDline ------------------------------------------------------------------------------------------------------------
    

    ! ----- struc_Geo_Coordinates ----------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_Coordinates
       
    	! UV/IR
    	real, dimension(:), allocatable :: xsc_x
        real, dimension(:), allocatable :: xsc_y
        real, dimension(:), allocatable :: xsc_z
        real, dimension(:), allocatable :: ysc_x
        real, dimension(:), allocatable :: ysc_y
        real, dimension(:), allocatable :: ysc_z
        real, dimension(:), allocatable :: zsc_x
        real, dimension(:), allocatable :: zsc_y
        real, dimension(:), allocatable :: zsc_z
        real, dimension(:), allocatable :: angle_xsc_localvert
        real, dimension(:), allocatable :: angle_xsc_proj
        real, dimension(:), allocatable :: p1_u
        real, dimension(:), allocatable :: p1_v
        real, dimension(:), allocatable :: p2_u
        real, dimension(:), allocatable :: p2_v
        real, dimension(:), allocatable :: p3_u
        real, dimension(:), allocatable :: p3_v
        real, dimension(:), allocatable :: p4_u
        real, dimension(:), allocatable :: p4_v
        real, dimension(:), allocatable :: p5_u
        real, dimension(:), allocatable :: p5_v
    
    END TYPE struc_Geo_Coordinates
    ! ----- END struc_Geo_Coordinates ------------------------------------------------------------------------------------------------------------
    
    
    ! ----- struc_Geo_TransMatrix ----------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo_TransMatrix
        
    	! UV/IR
    	real, dimension(:), allocatable :: x_ra
        real, dimension(:), allocatable :: x_dec
        real, dimension(:), allocatable :: y_ra
        real, dimension(:), allocatable :: y_dec
        real, dimension(:), allocatable :: z_ra
        real, dimension(:), allocatable :: z_dec
        
    END TYPE struc_Geo_TransMatrix
    ! ----- END struc_Geo_TransMatrix ------------------------------------------------------------------------------------------------------------
    

    ! ----- struc_Geo ----------------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_Geo
        
    	! UV/IR
    	type(struc_Geo_Record) Record
        type(struc_Geo_Spacecraft) Spacecraft
        type(struc_Geo_Coordinates) Coordinates
        type(struc_Geo_TransMatrix) TransMatrix
        
        ! UV ONLY
        type(struc_Geo_Band3LOSE) Band3
        type(struc_Geo_Band3LOSE) LOSE
        type(struc_Geo_BandCCDline) Band1
        type(struc_Geo_BandCCDline) Band2
        type(struc_Geo_BandCCDline) Band4
        type(struc_Geo_BandCCDline) Band5
        type(struc_Geo_BandCCDline) CCDLine    
        
        ! IR ONLY
        type(struc_Geo_Band3LOSE) IRFOV    
        
    END TYPE struc_Geo
    ! ----- END struc_Geo ------------------------------------------------------------------------------------------------------------------------

    
    ! ----- struc_1AFile -------------------------------------------------------------------------------------------------------------------------
    TYPE :: struc_1AFile
    
    	! UV/IR
        real, dimension(:,:,:), allocatable :: CleanData
        type(struc_Info) Info
        type(struc_Parameters) Parameters
        type(struc_GeoInfo) GeoInfo
        type(struc_Geo) Geo
        
    	! UV
        real, dimension(:,:,:), allocatable :: ErrData
        integer, dimension(:,:,:), allocatable :: Flag
        
        ! IR
        real, dimension(:,:,:), allocatable :: WAVELENGTH
        real, dimension(:,:,:), allocatable :: DC
        real, dimension(:,:,:), allocatable :: RAW
        type(struc_Time_of_Records) Time_of_Records
        
    END TYPE struc_1AFile;
    ! ----- END struc_1AFile ---------------------------------------------------------------------------------------------------------------------


! ----- END Structures declaration -----------------------------------------------------------------------------------------------------------------------------
! --------------------------------------------------------------------------------------------------------------------------------------------------------------

   
end module spica_struct_1a_mod_type

